use core::fmt::Write;
use heapless::{String, Vec};
use log::info;
use pico_website::unwrap;
use ringbuffer::{ConstGenericRingBuffer, RingBuffer};

use crate::socket::HttpResCode;

use super::App;

const MEMORY_SIZE: usize = 16;

pub struct ChatApp {
    res_buf: Vec<u8, 2048>,
    msgs: ConstGenericRingBuffer<Message, MEMORY_SIZE>,
    next_msg: u16,
}
impl ChatApp {
    pub fn new() -> Self {
        let mut msgs = ConstGenericRingBuffer::new();

        let mut aut0 = String::new();
        write!(&mut aut0, "rael").unwrap();
        let mut msg0 = String::new();
        write!(&mut msg0, "Prout...").unwrap();
        msgs.push(Message {
            author: aut0,
            content: msg0,
        });
        let mut aut1 = String::new();
        write!(&mut aut1, "josh").unwrap();
        let mut msg1 = String::new();
        write!(&mut msg1, "Salut !\nJe m'appelle...").unwrap();
        msgs.push(Message {
            author: aut1,
            content: msg1,
        });

        Self {
            res_buf: Vec::new(),
            msgs,
            next_msg: 2,
        }
    }
}
impl App for ChatApp {
    fn socket_name(&self) -> &'static str {
        "chat"
    }
    async fn handle_request<'a>(&'a mut self, path: &str) -> (HttpResCode, &'static str, &'a [u8]) {
        match path {
            "/" | "/index" | "/index.html" | "/chat" | "/chat.html" => {
                (HttpResCode::Ok, "html", include_bytes!("./chat.html"))
            }
            path => {
                let (path, args) = path.split_once('?').unwrap_or((path, ""));
                let mut load = None;
                let mut username = None;
                for arg in args.split('&') {
                    match arg.split_once('=') {
                        Some(("load", n)) => {
                            let n: u16 = match n.parse() {
                                Ok(v) => v,
                                Err(_) => return (HttpResCode::BadRequest, "", &[]),
                            };
                            if n > 0 {
                                load = Some(n);
                            }
                        }
                        Some(("username", u)) => {
                            if u.len() < 3 || u.len() > 16 {
                                return (HttpResCode::BadRequest, "", &[]);
                            }
                            username = Some(u);
                        }
                        _ => {}
                    }
                }
                if path == "/chat" && username.is_some() {
                    return (
                        HttpResCode::Ok,
                        "html",
                        "<div \
                            class=\"message\" \
                            hx-get=\"/chat/message/abs/0?load=\" \
                            hx-target=\"this\" \
                            hx-swap=\"outerHTML\" \
                            hx-trigger=\"load\" \
                        ></div>",
                    );
                } else if path.starts_with("/chat/message/abs/") && path.len() > 18 {
                    let msg_id: u16 = match path[18..].parse() {
                        Ok(n) => n,
                        Err(_) => return (HttpResCode::BadRequest, "", &[]),
                    };
                    info!("msg_id = {:?}", msg_id);
                    if msg_id >= self.next_msg {
                        return (HttpResCode::BadRequest, "", &[]);
                    }
                    if msg_id < self.next_msg.saturating_sub(MEMORY_SIZE as u16 + 1) {
                        return (HttpResCode::NoContent, "", &[]);
                    }
                    let msg = match self
                        .msgs
                        .get_signed((msg_id as isize) + 1 - (self.next_msg as isize))
                    {
                        Some(msg) => msg,
                        None => return (HttpResCode::NoContent, "", &[]),
                    };
                    self.res_buf.clear();
                    unwrap(write!(&mut self.res_buf, "<div class=\"message\"")).await;
                    if let Some(n) = load {
                        unwrap(write!(
                            &mut self.res_buf,
                            " hx-get=\"/chat/message/abs/{}?load={}\" \
                            hx-target=\"this\" \
                            hx-swap=\"afterend\" \
                            hx-trigger=\"load\"",
                            msg_id + 1,
                            n - 1
                        ))
                        .await;
                    }
                    unwrap(write!(
                        &mut self.res_buf,
                        "><b>{}</b>: {}</div>",
                        msg.author, msg.content
                    ))
                    .await;
                    // unwrap(write!(
                    //     &mut self.res_buf,
                    //     "<div class=\"message\" \
                    //         hx-get=\"/chat/message{}\" \
                    //         hx-target=\"next .message\" \
                    //         hx-swap=\"outerHTML\" \
                    //         hx-trigger=\"load\"\
                    //     >\
                    //         <b>{}</b>: {}\
                    //     </div>",
                    //     match load {
                    //         Some(target) => "hx-get=\"/chat/message{}\" \
                    //         hx-target=\"next .message\" \
                    //         hx-swap=\"outerHTML\" \
                    //         hx-trigger=\"load\"\"
                    //     }
                    //     (msg_num + 1),
                    //     msg.author,
                    //     msg.content
                    // ))
                    // .await;
                    return (HttpResCode::Ok, "html", &self.res_buf);
                } else {
                    (HttpResCode::NotFound, "", &[])
                }
            }
        }
    }
}

struct Message {
    author: String<16>,
    content: String<256>,
}
